/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.l2jfree.gameserver.network.clientpackets;

import com.l2jfree.gameserver.datatables.HennaTable;
import com.l2jfree.gameserver.model.actor.instance.L2PcInstance;
import com.l2jfree.gameserver.network.SystemMessageId;
import com.l2jfree.gameserver.network.serverpackets.HennaItemDrawInfo;
import com.l2jfree.gameserver.templates.item.L2Henna;

/**
 * RequestHennaItemDrawInfo
 * Format cd
 */
public final class RequestHennaItemDrawInfo extends L2GameClientPacket
{
	private static final String _C__BB_RequestHennaItemDrawInfo = "[C] bb RequestHennaItemDrawInfo";

	private int _symbolId;

	@Override
	protected void readImpl()
	{
		_symbolId = readD();
	}

	@Override
	protected void runImpl()
	{
		L2PcInstance activeChar = getClient().getActiveChar();
		if (activeChar == null || _symbolId == 0) // 0 = player closed window
			return;

		L2Henna template = HennaTable.getInstance().getTemplate(_symbolId);
		if (template == null)
		{
			requestFailed(SystemMessageId.SYMBOL_NOT_FOUND);
			return;
		}

		sendPacket(new HennaItemDrawInfo(template, activeChar));

		sendAF();
	}

	@Override
	public String getType()
	{
		return _C__BB_RequestHennaItemDrawInfo;
	}
}
